#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   ffmpegHelper.py
@Time    :   2018/12/17
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :   
'''

import os
import re
import shutil
import subprocess

import aigpy.netHelper as netHelper
import aigpy.pathHelper as pathHelper
import aigpy.threadHelper as threadHelper

from aigpy.progressHelper import ProgressTool

class FFmpegTool(object):
    def __init__(self, threadNum=50, mergerTimeout=None):
        """
        #Func    :   初始化             
        #Param   :   threadNum     [in] 线程数          
        #Param   :   mergerTimeout [in] 超时 秒     
        #Return  :   True/False             
        """
        self.thread        = threadHelper.ThreadTool(threadNum)
        self.waitCount     = 0
        self.completeCount = 0
        self.progress      = None
        self.mergerTimeout = mergerTimeout
        return
    
    def __thradfunc_dl(self, url, filepath, retrycount):
        check = False
        try:
            while retrycount > 0:
                retrycount = retrycount - 1
                check = netHelper.downloadFile(url, filepath, 30)
                if check:
                    break
        except:
            pass
        self.completeCount = self.completeCount + 1
        if self.progress is not None:
            self.progress.step()
        return

    
    def __parseM3u8(self, url):
        content = netHelper.downloadString(url, None)
        pattern = re.compile(r"(?<=http).+?(?=\\n)")
        plist   = pattern.findall(str(content))
        urllist = []
        for item in plist:
            urllist.append("http"+item)
        return urllist

    def __process(self, cmd, retrycount, showshell, filename):
        while retrycount >= 0:
            retrycount -= 1
            try:
                if showshell:
                    res = subprocess.call(cmd, timeout=self.mergerTimeout, shell=True)
                else:
                    res = subprocess.call(cmd, timeout=self.mergerTimeout, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                if res == 0:
                    return True
            except:
                pass
            pathHelper.remove(filename)
        return False

    def mergerByM3u8_Multithreading(self, url, filepath, showprogress=False, showshell=False):
        """
        #Func    :   多线程下载并合并文件(使用M3u8的url)        
        #Param   :   url             [in] 链接
        #Param   :   filepath        [in] 目标文件名
        #Param   :   showprogress    [in] 是否显示进度条
        #Param   :   showshell       [in] 是否显示cmd信息
        #Return  :   True/False
        """
        try:
            # Get urllist
            urllist = self.__parseM3u8(url)
            if len(urllist) <= 0:
                return False

            # Creat tmpdir
            path    = pathHelper.getDirName(filepath)
            tmpPath = pathHelper.getDiffTmpPathName(path)
            if pathHelper.mkdirs(tmpPath) is False:
                return False
            
            # Progress
            self.progress = None
            if showprogress:
                self.progress = ProgressTool(len(urllist))

            # Download files
            index              = 0
            allpath            = []
            self.waitCount     = len(urllist)
            self.completeCount = 0
            for item in urllist:
                index = index + 1
                path = tmpPath + '/' + str(index) + ".ts"
                allpath.append(path)
                if os.path.exists(path):
                    os.remove(path)
                self.thread.start(self.__thradfunc_dl, item, path, 3)
            self.thread.waitAll()
            ret = self.mergerByFiles(allpath, filepath, showshell)
            shutil.rmtree(tmpPath)
            return ret
        except:
            return False

    def mergerByM3u8(self, url, filepath, showshell=False):
        """
        #Func    :   合并文件(使用M3u8的url)        
        #Param   :   url         [in] 链接       
        #Param   :   filepath    [in] 目标文件名            
        #Param   :   showshell   [in] 是否显示cmd信息              
        #Return  :   True/False 
        """
        res = -1
        try:
            filepath = os.path.abspath(filepath)
            cmd = "ffmpeg -safe 0 -i " + url + " -c copy -bsf:a aac_adtstoasc \"" + filepath + "\""
            res = self.__process(cmd, 3, showshell, filepath)
        except:
            pass
        return res

    def mergerByFiles(self, srcfilepaths, filepath, showshell=False):
        """
        #Func    :   合并文件       
        #Param   :   srcfilepaths   [in] 文件名数组     
        #Param   :   filepath       [in] 目标文件名     
        #Param   :   showshell      [in] 是否显示cmd信息        
        #Return  :   True/False 
        """
        filepath = os.path.abspath(filepath)
        res     = -1
        tmpfile = filepath + "TMP.txt"
        paths   = srcfilepaths
        group   = None
        groupnum= 50
        try:
            # 先分组合并，再一起合并
            if len(srcfilepaths) > groupnum:
                dirName = pathHelper.getDirName(filepath)
                group   = pathHelper.getDiffTmpPathName(dirName)
                if pathHelper.mkdirs(group) is False:
                    return False
                
                newPaths = []
                array    = [srcfilepaths[i:i+groupnum] for i in range(0, len(srcfilepaths), groupnum)]
                for index, item in enumerate(array):
                    file = group + '/' + str(index) + ".mp4"
                    newPaths.append(file)
                    if self.mergerByFiles(item, file, showshell) is False:
                        return False
                paths = newPaths

            # 创建临时文件,将要合并的文件名列表写入
            with open(tmpfile, 'w') as fd:
                for item in paths:
                    item = os.path.abspath(item)
                    fd.write('file \'' + item + '\'\n')


            # 调用ffmpeg进行合并
            cmd = "ffmpeg -f concat -safe 0 -i \"" + tmpfile + "\" -c copy \"" + filepath + "\""
            res = self.__process(cmd, 3, showshell, filepath)
        except:
            pass

        if os.access(tmpfile,0):
            os.remove(tmpfile)
        if group is not None:
            shutil.rmtree(group)
        return res


# tool = FFmpegTool(1)
# array = []
# array.append('E://7//Video//Tmp0//1.ts')
# array.append('E://7//Video//Tmp0//2.ts')
# array.append('E://7//Video//Tmp0//3.ts')
# tool.mergerByFiles(array, 'E://7//Video//test.mp4')




import socks
import requests
import sys
import time

sys.path.append('./')
import aigpy.zipHelper as zipHelper
from aigpy.updateHelper import updateTool
import aigpy.ffmpegHelper as ffmpegHelper
from aigpy.serverHelper import ServerTool

def func(httpreq):
    i = 0
    return '111'

if __name__ == '__main__':
    rootDir = 'C:\\Users\\huangyue\\Downloads\\登录界面模板\\后台\\main\\'
    server = ServerTool(rootDir, rootDir)

    server.start('127.0.0.1', 9999)
    while True:
        pass
    plist = []
    plist.append("e:\\test\\1.ini")
    plist.append("e:\\test\\2.ini")
    zipHelper.zip(plist, "e:\\zip.zip")
    zipHelper.unzip("e:\\zip.zip", "e:\\test2")
    print('===VERSION 1.0.0.1===')
    print(sys.argv)

    tf = updateTool(__file__, 'tet', 'http://144.34.241.208//test', '1.0.0.1', 'aigpy.exe', None)
    print(tf.curPath)

    if tf.go() == False:
        print('==OVER!==')
        time.sleep(100)
#    def __run(self, cmd):
#         p = subprocess.Popen(cmd,
#             stdout=subprocess.PIPE, 
#             stderr=subprocess.PIPE,
#             close_fds=True)
#         out, err = p.communicate()
#         out = out.decode('utf8')
#         err = err.decode('utf8')
#         status = p.wait()
#         # check exit status
#         if not os.WIFEXITED(status) or os.WEXITSTATUS(status):
#             if not re.match(r'(iptables|ip6tables): Chain already exists', err):
#                 raise IptablesError(cmd, err)
#         return out



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   pathHelper.py
@Time    :   2018/12/17
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :   
'''

import os
import shutil

def getDiffTmpPathName(basePath):
    """
    #Func    :   获取临时文件名‘Tmp+X’              
    #Param   :   basePath        [in] 基础路径             
    #Return  :   string            
    """
    count = 0
    basePath = basePath.replace("\\", "/")
    basePath = basePath.strip()
    basePath = basePath.rstrip("/")
    path = basePath + '/Tmp' + str(count)
    while os.path.exists(path):
        count = count + 1
        path  = basePath + '/Tmp' + str(count)
    return path

def mkdirs(path):
    """
    #Func    :   递归生成目录                
    #Param   :   path        [in] 基础路径                
    #Return  :   True/False         
    """
    path = path.replace("\\", "/")
    path = path.strip()
    path = path.rstrip("/")
    if not os.path.exists(path):
        os.makedirs(path)
        return True
    return False

def remove(path):
    """
    #Func    :   删除目录或文件                      
    #Param   :   basePath        [in] 基础路径                
    #Return  :   True/False           
    """
    try:
        if(os.path.exists(path) is False):
            return True
        if os.path.isfile(path) is True:
            os.remove(path)
        if os.path.isdir(path) is True:
            shutil.rmtree(path)
        return True
    except:
        return False


def copyFile(srcfile, dstfile):
    """
    #Func    :   复制文件                         
    #Param   :   srcfile        [in] 源文件路径                  
    #Param   :   dstfile        [in] 目标文件路径                        
    #Return  :   True/False       
    """
    if not os.path.isfile(srcfile):
        return False
    else:
        fpath, fname = os.path.split(dstfile)  
        if not os.path.exists(fpath):
            os.makedirs(fpath) 
        shutil.copyfile(srcfile, dstfile) 
    return True


def replaceLimitChar(path, newChar):
    """
    #Func    :   移除路径名中不符合的字符                                         
    #Param   :   path           [in] 路径名                                 
    #Param   :   newChar        [in] 要替换的字符                               
    #Return  :   string            
    """
    if path is None:
        return ""
    if newChar is None:
        newChar = ''
    path = path.replace(':', newChar)
    path = path.replace('/', newChar)
    path = path.replace('?', newChar)
    path = path.replace('<', newChar)
    path = path.replace('>', newChar)
    path = path.replace('|', newChar)
    path = path.replace('\\', newChar)
    path = path.replace('*', newChar)
    path = path.replace('\"', newChar)
    return path

def getDirName(filepath):
    """
    #Func    :   获取目录名                                                  
    #Param   :   filepath      [in] 路径文件名                                          
    #Return  :   string              
    """
    filepath = filepath.replace('\\', '/')
    index    = filepath.rfind('/')
    if index == -1:
        return './'
    return filepath[0:index+1]

def getFileName(filepath):
    """
    #Func    :   获取文件名                                                           
    #Param   :   filepath      [in] 路径文件名                                          
    #Return  :   string              
    """
    filepath = filepath.replace('\\', '/')
    index = filepath.rfind('/')
    if index == -1:
        return filepath
    return filepath[index+1:len(filepath)]

def getFileNameWithoutExtension(filepath):
    """ 
    #Func    :   获取不带扩展名的文件名                                                    
    #Param   :   filepath      [in] 路径文件名                                           
    #Return  :   string               
    """
    filepath = getFileName(filepath)
    index = filepath.rfind('.')
    if index == -1:
        return filepath
    return filepath[0:index]

def getFileExtension(filepath):
    """
    #Func    :   获取扩展名(如.txt)                                                              
    #Param   :   filepath      [in] 路径文件名                                              
    #Return  :   string                 
    """
    filepath = getFileName(filepath)
    index = filepath.rfind('.')
    if index == -1:
        return
    return filepath[index:len(filepath)]

def getDirSize(path):
    """
    #Func    :   获取目录大小                                                             
    #Param   :   filepath      [in] 路径名                                               
    #Return  :   整数                     
    """
    try:
        if os.path.isdir(path) is False:
            return 0
        size = 0
        for root, dirs, files in os.walk(path):
            size += sum([os.path.getsize(os.path.join(root, name)) for name in files])
        return size
    except:
        return 0

def getDirFiles(path):
    """
    #Func    :   获取文件下的全部路径文件名                                                             
    #Param   :   filepath      [in] 路径名                                                
    #Return  :   字符串数组                  
    """
    try:
        if os.path.isdir(path) is False:
            return []
        ret = []
        for root, dirs, files in os.walk(path):
            root = root.replace('\\','/')
            for item in files:
                ret.append(root + '/' + item)
        return ret
    except:
        return []



from setuptools import setup, find_packages
setup(
    name = 'aigpy',
    version='2019.5.8.1',
    license = "MIT Licence",
    description = "Python Common Tool",

    author = 'YaronH',
    author_email = "yaronhuang@qq.com",

    packages = find_packages(),
    include_package_data = True,
    platforms = "any",
    install_requires=["requests", "configparser", "futures", "reportlab", "Pillow"],
    # entry_points={'console_scripts': [
    #     'tidal-dl = tidal_dl:main', ]}
)
